@@ -16,9 +16,10 @@
   GNU General Public License for more details.
 
 */
-/* $Id: client_ftp.c,v 1.9 2002/02/03 17:21:14 priikone Exp $ */
+/* $Id: client_ftp.c,v 1.13 2002/03/19 07:13:13 priikone Exp $ */
 
-#include "clientlibincludes.h"
+#include "silcincludes.h"
+#include "silcclient.h"
 #include "client_internal.h"
 
 static int
@@ -33,7 +34,7 @@ static void silc_client_ftp_start_key_agreement(SilcClientFtpSession session,
 
 /* File transmission session */
 struct SilcClientFtpSessionStruct {
-  uint32 session_id;
+  SilcUInt32 session_id;
   SilcClient client;
   SilcClientConnection conn;
   SilcClientEntry client_entry;
@@ -42,7 +43,7 @@ struct SilcClientFtpSessionStruct {
   SilcBuffer packet;
 
   char *hostname;
-  uint16 port;
+  SilcUInt16 port;
   int listener;
 
   SilcClientFileMonitor monitor;
@@ -55,8 +56,8 @@ struct SilcClientFtpSessionStruct {
 
   SilcSFTPHandle dir_handle;
   SilcSFTPHandle read_handle;
-  uint64 filesize;
-  uint64 read_offset;
+  SilcUInt64 filesize;
+  SilcUInt64 read_offset;
   int fd;
 };
 
@@ -158,8 +159,7 @@ silc_client_connect_to_client(SilcClient client,
 /* SFTP packet send callback. This will use preallocated buffer to avoid
    reallocation of outgoing data buffer everytime. */
 
-static void silc_client_ftp_send_packet(SilcSocketConnection sock,
-					SilcBuffer packet, void *context)
+static void silc_client_ftp_send_packet(SilcBuffer packet, void *context)
 {
   SilcClientFtpSession session = (SilcClientFtpSession)context;
   SilcClient client = session->client;
@@ -182,8 +182,9 @@ static void silc_client_ftp_send_packet(SilcSocketConnection sock,
 		     SILC_STR_END);
 
   /* Send the packet immediately */
-  silc_client_packet_send(client, sock, SILC_PACKET_FTP, NULL, 0, NULL, NULL,
-			  session->packet->data, session->packet->len, TRUE);
+  silc_client_packet_send(client, session->sock, SILC_PACKET_FTP, NULL, 
+			  0, NULL, NULL, session->packet->data, 
+			  session->packet->len, TRUE);
 
   /* Clear buffer */
   session->packet->data = session->packet->tail = session->packet->head;
@@ -221,7 +222,7 @@ static void silc_client_ftp_monitor(SilcSFTP sftp,
 static void silc_client_ftp_data(SilcSFTP sftp,
 				 SilcSFTPStatus status,
 				 const unsigned char *data,
-				 uint32 data_len,
+				 SilcUInt32 data_len,
 				 void *context)
 {
   SilcClientFtpSession session = (SilcClientFtpSession)context;
@@ -264,7 +265,8 @@ static void silc_client_ftp_data(SilcSFTP sftp,
 
   /* Read more, until EOF is received */
   session->read_offset += data_len;
-  silc_sftp_read(sftp, session->read_handle, session->read_offset, 64512,
+  silc_sftp_read(sftp, session->read_handle, session->read_offset, 
+                 SILC_PACKET_MAX_LEN - 1024,
 		 silc_client_ftp_data, session);
 
   /* Call monitor callback */
@@ -330,7 +332,8 @@ static void silc_client_ftp_open_handle(SilcSFTP sftp,
   session->read_handle = handle;
 
   /* Now, start reading the file */
-  silc_sftp_read(sftp, session->read_handle, session->read_offset, 64512,
+  silc_sftp_read(sftp, session->read_handle, session->read_offset,
+                 SILC_PACKET_MAX_LEN - 1024,
 		 silc_client_ftp_data, session);
 
   /* Call monitor callback */
@@ -489,14 +492,12 @@ SILC_TASK_CALLBACK(silc_client_ftp_key_agreement_final)
   if (!session->server) {
     /* If we are the SFTP client then start the SFTP session and retrieve
        the info about the file available for download. */
-    session->sftp = silc_sftp_client_start(conn->sock,
-					   silc_client_ftp_send_packet,
-					   session, 
-					   silc_client_ftp_version, session);
+    session->sftp = silc_sftp_client_start(silc_client_ftp_send_packet,
+					   session, silc_client_ftp_version, 
+					   session);
   } else {
     /* Start SFTP server */
-    session->sftp = silc_sftp_server_start(conn->sock,
-					   silc_client_ftp_send_packet,
+    session->sftp = silc_sftp_server_start(silc_client_ftp_send_packet,
 					   session, session->fs);
 
     /* Monitor transmission */
@@ -779,10 +780,10 @@ silc_client_file_send(SilcClient client,
 		      SilcClientFileMonitor monitor,
 		      void *monitor_context,
 		      const char *local_ip,
-		      uint32 local_port,
+		      SilcUInt32 local_port,
 		      SilcClientEntry client_entry,
 		      const char *filepath,
-		      uint32 *session_id)
+		      SilcUInt32 *session_id)
 {
   SilcClientFtpSession session;
   SilcBuffer keyagr, ftp;
@@ -890,7 +891,7 @@ silc_client_file_receive(SilcClient client,
 			 SilcClientConnection conn,
 			 SilcClientFileMonitor monitor,
 			 void *monitor_context,
-			 uint32 session_id)
+			 SilcUInt32 session_id)
 {
   SilcClientFtpSession session;
   SilcBuffer keyagr, ftp;
@@ -972,7 +973,7 @@ silc_client_file_receive(SilcClient client,
 
 SilcClientFileError silc_client_file_close(SilcClient client,
 					   SilcClientConnection conn,
-					   uint32 session_id)
+					   SilcUInt32 session_id)
 {
   SilcClientFtpSession session;
 
@@ -1004,7 +1005,7 @@ SilcClientFileError silc_client_file_close(SilcClient client,
 static void silc_client_ftp_resolve_cb(SilcClient client,
 				       SilcClientConnection conn,
 				       SilcClientEntry *clients,
-				       uint32 clients_count,
+				       SilcUInt32 clients_count,
 				       void *context)
 {
   SilcPacketContext *packet = (SilcPacketContext *)context;
@@ -1012,7 +1013,7 @@ static void silc_client_ftp_resolve_cb(SilcClient client,
   SilcKeyAgreementPayload payload = NULL;
   SilcClientEntry client_entry;
   char *hostname;
-  uint16 port;
+  SilcUInt16 port;
 
   SILC_LOG_DEBUG(("Start"));
 
@@ -1093,7 +1094,7 @@ void silc_client_ftp(SilcClient client,
 		     SilcPacketContext *packet)
 {
   SilcClientConnection conn = (SilcClientConnection)sock->user_data;
-  uint8 type;
+  SilcUInt8 type;
   int ret;
 
   SILC_LOG_DEBUG(("Start"));
