@@ -16,12 +16,13 @@
   GNU General Public License for more details.
 
 */
-/* $Id: client_notify.c,v 1.32 2002/01/31 20:30:25 priikone Exp $ */
+/* $Id: client_notify.c,v 1.36 2002/03/16 19:09:33 priikone Exp $ */
 /* This file includes the Notify packet handling. Notify packets are
    important packets sent by the server. They tell different things to the
    client such as nick changes, mode changes etc. */
 
-#include "clientlibincludes.h"
+#include "silcincludes.h"
+#include "silcclient.h"
 #include "client_internal.h"
 
 typedef struct {
@@ -98,13 +99,13 @@ void silc_client_notify_by_server(SilcClient client,
   SilcClientID *client_id = NULL;
   SilcChannelID *channel_id = NULL;
   SilcServerID *server_id = NULL;
-  SilcClientEntry client_entry;
-  SilcClientEntry client_entry2;
+  SilcClientEntry client_entry = NULL;
+  SilcClientEntry client_entry2 = NULL;
   SilcChannelEntry channel;
   SilcChannelUser chu;
   SilcServerEntry server;
   unsigned char *tmp;
-  uint32 tmp_len, mode;
+  SilcUInt32 tmp_len, mode;
 
   SILC_LOG_DEBUG(("Start"));
 
@@ -645,7 +646,7 @@ void silc_client_notify_by_server(SilcClient client,
       break;
 
     /* Save the mode */
-    chu = silc_client_on_channel(channel, client_entry);
+    chu = silc_client_on_channel(channel, client_entry2);
     if (chu)
       chu->mode = mode;
 
@@ -743,21 +744,20 @@ void silc_client_notify_by_server(SilcClient client,
 
     /* Get the kicker */
     tmp = silc_argument_get_arg_type(args, 3, &tmp_len);
-    if (!tmp)
-      goto out;
+    if (tmp) {
+      client_id = silc_id_payload_parse_id(tmp, tmp_len);
+      if (!client_id)
+	goto out;
 
-    client_id = silc_id_payload_parse_id(tmp, tmp_len);
-    if (!client_id)
-      goto out;
-
-    /* Find kicker's client entry and if not found resolve it */
-    client_entry2 = silc_client_get_client_by_id(client, conn, client_id);
-    if (!client_entry2) {
-      silc_client_notify_by_server_resolve(client, conn, packet, client_id);
-      goto out;
-    } else {
-      if (client_entry2 != conn->local_entry)
-	silc_client_nickname_format(client, conn, client_entry2);
+      /* Find kicker's client entry and if not found resolve it */
+      client_entry2 = silc_client_get_client_by_id(client, conn, client_id);
+      if (!client_entry2) {
+	silc_client_notify_by_server_resolve(client, conn, packet, client_id);
+	goto out;
+      } else {
+	if (client_entry2 != conn->local_entry)
+	  silc_client_nickname_format(client, conn, client_entry2);
+      }
     }
 
     /* Get comment */
@@ -822,7 +822,7 @@ void silc_client_notify_by_server(SilcClient client,
        * from channels as they quit as well.
        */
       SilcClientEntry *clients = NULL;
-      uint32 clients_count = 0;
+      SilcUInt32 clients_count = 0;
       int i;
 
       SILC_LOG_DEBUG(("Notify: SIGNOFF"));
