@@ -1,23 +1,22 @@
 /*
 
-  silcmemory.c
+  silcmemory.c 
 
-  Author: Pekka Riikonen <priikone@poseidon.pspt.fi>
+  Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1999 - 2000 Pekka Riikonen
+  Copyright (C) 1999 - 2002 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-  
+  the Free Software Foundation; version 2 of the License.
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
 */
-/* $Id: silcmemory.c,v 1.4 2001/09/04 11:52:09 priikone Exp $ */
+/* $Id: silcmemory.c,v 1.5 2002/02/17 20:37:52 priikone Exp $ */
 
 #include "silcincludes.h"
 
@@ -49,3 +48,12 @@ void silc_free(void *ptr)
 {
   free(ptr);
 }
+
+void *silc_memdup(const void *ptr, size_t size)
+{
+  unsigned char *addr = silc_malloc(size + 1);
+  assert(addr != NULL);
+  memcpy((void *)addr, ptr, size);
+  addr[size] = '\0';
+  return (void *)addr;
+}
